//---------------------------------------------------------------------------//
//
//	Graphics - Texture
//
//	Date : 2008/04/10
//
//---------------------------------------------------------------------------//

#include "graphics_std.h"
#include "renderer.h"
#include "texture.h"

CTexture::CTexture()
: m_pTexture(NULL)
{
	m_nMipLevels = 0;
	m_Pool = D3DPOOL_DEFAULT;
	m_Format = D3DFMT_UNKNOWN;
	m_dwTransparent = 0;

	m_dwFilter = D3DX_DEFAULT;
	m_dwMipFilter = D3DX_DEFAULT;
	m_nWidth = 1;
	m_nHeight = 1;
}

CTexture::~CTexture()
{
	Release();
}

bool CTexture::Create(UINT nWidth, UINT nHeight, D3DFORMAT Format, UINT nMipLevels, D3DPOOL Pool)
{
	ASSERT(m_pTexture == NULL && (Pool == D3DPOOL_SYSTEMMEM || Pool == D3DPOOL_MANAGED) && nWidth > 0 && nHeight > 0);

	LPDIRECT3DDEVICE9 pDevice = GetRenderer()->GetDevice();
	
	HRESULT hr = pDevice->CreateTexture(nWidth, nHeight, nMipLevels, D3DUSAGE_AUTOGENMIPMAP, Format, Pool, &m_pTexture, NULL);
	if (hr != D3D_OK)
	{
		return false;
	}

	m_nMipLevels = nMipLevels;
	m_Pool = Pool;

	UpdateInfo();

	return true;
}
bool CTexture::Create(UINT nWidth, UINT nHeight, DWORD dwUsage, D3DFORMAT Format, UINT nMipLevels, D3DPOOL Pool)
{
	ASSERT(m_pTexture == NULL && (Pool == D3DPOOL_SYSTEMMEM || Pool == D3DPOOL_MANAGED || Pool == D3DPOOL_DEFAULT) && nWidth > 0 && nHeight > 0);

	LPDIRECT3DDEVICE9 pDevice = GetRenderer()->GetDevice();
	
	HRESULT hr = pDevice->CreateTexture(nWidth, nHeight, nMipLevels, dwUsage, Format, Pool, &m_pTexture, NULL);
	if (hr != D3D_OK)
	{
		return false;
	}

	m_nMipLevels = nMipLevels;
	m_Pool = Pool;

	UpdateInfo();

	return true;
}

bool CTexture::Lock(D3DLOCKED_RECT& Locked, bool bReadOnly)
{
	ASSERT(m_pTexture != NULL && (m_Pool == D3DPOOL_SYSTEMMEM || m_Pool == D3DPOOL_MANAGED));

	DWORD dwFlags = 0;
	if (bReadOnly)
	{
		dwFlags |= D3DLOCK_READONLY;
	}

	HRESULT hr = m_pTexture->LockRect(0, &Locked, NULL, dwFlags);
	if (hr != D3D_OK)
	{
		return false;
	}

	return true;
}

void CTexture::Unlock()
{
	ASSERT(m_pTexture != NULL && (m_Pool == D3DPOOL_SYSTEMMEM || m_Pool == D3DPOOL_MANAGED));
	m_pTexture->UnlockRect(0);

	if (m_nMipLevels != 1)
	{
		//m_pTexture->GenerateMipSubLevels();
	}
}

bool CTexture::Load(const tstring& strFilename, UINT nMipLevels, D3DFORMAT Format, D3DPOOL Pool, DWORD dwFilter, DWORD dwMipFilter, DWORD dwTransparent)
{
	ASSERT(m_pTexture == NULL);

	LPDIRECT3DDEVICE9 pDevice = GetRenderer()->GetDevice();

	if (Format == D3DFMT_UNKNOWN)
	{
		if (GetRenderer()->GetBackBufferFormat() == D3DFMT_R5G6B5)
		{
			Format = D3DFMT_A1R5G5B5;
		}
		else
		{
			Format = D3DFMT_A8R8G8B8;
		}
	}

	HRESULT hr = D3DXCreateTextureFromFileEx(pDevice, strFilename.c_str(), 
											D3DX_DEFAULT, D3DX_DEFAULT, nMipLevels, 
											0, Format, Pool, 
											dwFilter, dwMipFilter, dwTransparent, 
											NULL, NULL, &m_pTexture);

	if (hr != D3D_OK)
	{
		return false;
	}

	m_strFilename = strFilename;
	m_nMipLevels = nMipLevels;
	m_Pool = Pool;
	m_dwFilter = dwFilter;
	m_dwMipFilter = dwMipFilter;
	m_dwTransparent = dwTransparent;

	UpdateInfo();

	return true;
}

void CTexture::Release()
{
	SAFE_RELEASE(m_pTexture);
	m_nMipLevels = 0;
	m_Pool = D3DPOOL_DEFAULT;
	m_dwTransparent = 0;
	m_dwFilter = D3DX_DEFAULT;
	m_dwMipFilter = D3DX_DEFAULT;
}

bool CTexture::SetTexture(DWORD dwStage)
{
	if(m_pTexture) 
		GetRenderer()->GetDevice()->SetTexture(dwStage, m_pTexture);

	return true;
}

LPDIRECT3DTEXTURE9 CTexture::GetTexture()
{
	ASSERT(m_pTexture);
	return m_pTexture;;
}

void CTexture::Disable()
{
	if (m_Pool == D3DPOOL_DEFAULT)
	{
		SAFE_RELEASE(m_pTexture);
	}
}

void CTexture::Restore()
{
	if (m_Pool == D3DPOOL_DEFAULT)
	{
		assert(!m_strFilename.empty());

		if (Load(m_strFilename, m_nMipLevels, m_Format, m_Pool, m_dwFilter, m_dwMipFilter, m_dwTransparent) == false)
		{
			ASSERT_MSG(0, "CTexture::Restore() - restoring texture is failed.!!");
			return;
		}
	}
}

//! ؽ  ƮѴ.(, , )
bool CTexture::UpdateInfo()
{
	D3DSURFACE_DESC d3dsd;

	if (!m_pTexture || FAILED(m_pTexture->GetLevelDesc(0, &d3dsd))) 
		return false;

	m_nWidth = d3dsd.Width;
	m_nHeight = d3dsd.Height;
	m_Format = d3dsd.Format;

	return true;
}